@@ -3,15 +3,27 @@
 //SPDX-License-Identifier: GPL-3.0-or-later
 #include "powerworker.h"
 #include "powermodel.h"
-#include "widgets/utils.h"
+#include "utils.h"
 
 #include <QProcessEnvironment>
 #include <QFutureWatcher>
 #include <QtConcurrent>
+#include <QRegularExpression>
 
 #define POWER_CAN_SLEEP "POWER_CAN_SLEEP"
 #define POWER_CAN_HIBERNATE "POWER_CAN_HIBERNATE"
 
+#define BATTERY_LOCKDELAY_NAME "batteryLockDelay"
+#define BATTERY_SLEEPDELAY_NAME "batterySleepDelay"
+#define BATTERY_SBDELAY_NAME "batteryScreenBlackDelay"
+#define LINEPOWER_LOCKDELAY_NAME "linePowerLockDelay"
+#define LINEPOWER_SLEEPDELAY_NAME "linePowerSleepDelay"
+#define LINEPOWER_SBDELAY_NAME "linePowerScreenBlackDelay"
+
+#define SHOW_HIBERNATE_NAME "showHibernate"
+#define SHOW_SHUTDOWN_NAME "showShutdown"
+#define SHOW_SUSPEND_NAME "showSuspend"
+
 static const QStringList DCC_CONFIG_FILES {
     "/etc/deepin/dde-control-center.conf",
     "/usr/share/dde-control-center/dde-control-center.conf"
@@ -21,6 +33,8 @@ PowerWorker::PowerWorker(PowerModel *model, QObject *parent)
     : QObject(parent)
     , m_powerModel(model)
     , m_powerDBusProxy(new PowerDBusProxy(this))
+    , m_cfgDock(DConfig::create("org.deepin.dde.tray-loader", "org.deepin.dde.dock.plugin.power", QString(), this))
+    , m_cfgPower(DConfig::create("org.deepin.dde.control-center", "org.deepin.dde.control-center.power", QString(), this))
 {
     connect(m_powerDBusProxy, &PowerDBusProxy::noPasswdLoginChanged, m_powerModel, &PowerModel::setNoPasswdLogin);
     connect(m_powerDBusProxy, &PowerDBusProxy::ScreenBlackLockChanged, m_powerModel, &PowerModel::setScreenBlackLock);
@@ -55,9 +69,81 @@ PowerWorker::PowerWorker(PowerModel *model, QObject *parent)
     connect(m_powerDBusProxy, &PowerDBusProxy::LowPowerAutoSleepThresholdChanged, m_powerModel, &PowerModel::setLowPowerAutoSleepThreshold);
     //-------------------------------------------------------
     connect(m_powerDBusProxy, &PowerDBusProxy::ModeChanged, m_powerModel, &PowerModel::setPowerPlan);
+    connect(m_powerDBusProxy, &PowerDBusProxy::BatteryCapacityChanged, m_powerModel, &PowerModel::setBatteryCapacity);
+
+    connect(m_cfgDock, &DConfig::valueChanged, [this] (const QString &key) {
+        if ("showTimeToFull" == key) {
+            m_powerModel->setShowBatteryTimeToFull(m_cfgDock->value("showTimeToFull").toBool());
+        }
+    });
+
+    connect(m_cfgPower, &DConfig::valueChanged, [this](const QString &key) {
+        if (key == BATTERY_LOCKDELAY_NAME) {
+            readConfig(BATTERY_LOCKDELAY_NAME,
+                std::bind(&PowerModel::setBatteryLockDelayModel, m_powerModel, std::placeholders::_1));
+        } else if (key == BATTERY_SLEEPDELAY_NAME) {
+            readConfig(BATTERY_SLEEPDELAY_NAME,
+                std::bind(&PowerModel::setBatterySleepDelayModel, m_powerModel, std::placeholders::_1));
+        } else if (key == BATTERY_SBDELAY_NAME) {
+            readConfig(BATTERY_SBDELAY_NAME,
+                std::bind(&PowerModel::setBatteryScreenBlackDelayModel, m_powerModel, std::placeholders::_1));
+        } else if (key == LINEPOWER_LOCKDELAY_NAME) {
+            readConfig(LINEPOWER_LOCKDELAY_NAME,
+                std::bind(&PowerModel::setLinePowerLockDelayModel, m_powerModel, std::placeholders::_1));
+        } else if (key == LINEPOWER_SLEEPDELAY_NAME) {
+            readConfig(LINEPOWER_SLEEPDELAY_NAME,
+                std::bind(&PowerModel::setLinePowerSleepDelayModel, m_powerModel, std::placeholders::_1));
+        } else if (key == LINEPOWER_SBDELAY_NAME) {
+            readConfig(LINEPOWER_SBDELAY_NAME,
+                std::bind(&PowerModel::setLinePowerScreenBlackDelayModel, m_powerModel, std::placeholders::_1));
+        } else if (key == SHOW_HIBERNATE_NAME) {
+            readConfig(SHOW_HIBERNATE_NAME,
+                std::bind(&PowerModel::setHibernate, m_powerModel, std::placeholders::_1));
+        } else if (key == SHOW_SHUTDOWN_NAME) {
+            readConfig(SHOW_SHUTDOWN_NAME,
+                std::bind(&PowerModel::setShutdown, m_powerModel, std::placeholders::_1));
+        } else if (key == SHOW_SUSPEND_NAME) {
+            readConfig(SHOW_SUSPEND_NAME,
+                std::bind(&PowerModel::setSuspend, m_powerModel, std::placeholders::_1));
+        }
+    });
 
     // init base property
     m_powerModel->setHaveBettary(m_powerDBusProxy->hasBattery());
+
+    active();
+}
+
+QVariantList PowerWorker::converToDataMap(const QStringList& conf)
+{
+    QVariantList dataMap;
+    for(const QString& numStr : conf) {
+        int num;
+        if (numStr.isEmpty()) {
+            qWarning() << "Convert to num failed, config is empty";
+            continue;
+        }
+        bool ok;
+        num = numStr.mid(0, numStr.length() - 1).toInt(&ok);
+        if (!ok) {
+            qWarning() << "Convert to num failed, can't change num to int";
+            num = 0;
+        }
+        if (numStr.contains("m")) {
+            num = num * 60;
+        } else if (numStr.contains("h")) {
+            num = num * 3600;
+        }
+        QVariantMap map;
+        map["text"] = numStr;
+        map["value"] = num;
+        dataMap.push_back(map);
+    }
+    QVariantMap map;
+    map["text"] = tr("Never");
+    map["value"] = 0;
+    dataMap.push_back(map);
+    return dataMap;
 }
 
 void PowerWorker::active()
@@ -81,6 +167,9 @@ void PowerWorker::active()
     m_powerModel->setBatteryPressPowerBtnAction(m_powerDBusProxy->batteryPressPowerBtnAction());
     m_powerModel->setBatteryLidClosedAction(m_powerDBusProxy->batteryLidClosedAction());
     m_powerModel->setPowerPlan(m_powerDBusProxy->mode());
+    m_powerModel->setBatteryCapacity(m_powerDBusProxy->batteryCapacity());
+    m_powerModel->setNoPasswdLogin(m_powerDBusProxy->noPasswdLogin());
+    m_powerModel->setShowBatteryTimeToFull(m_cfgDock->value("showTimeToFull").toBool());
 
     m_powerModel->setNoPasswdLogin(m_powerDBusProxy->noPasswdLogin());
 
@@ -129,6 +218,46 @@ void PowerWorker::active()
     canHibernateWatcher->setFuture(QtConcurrent::run([=] {
         return m_powerDBusProxy->login1ManagerCanHibernate();
     }));
+
+    readConfig(BATTERY_LOCKDELAY_NAME, std::bind(&PowerModel::setBatteryLockDelayModel, m_powerModel, std::placeholders::_1));
+    readConfig(BATTERY_SLEEPDELAY_NAME, std::bind(&PowerModel::setBatterySleepDelayModel, m_powerModel, std::placeholders::_1));
+    readConfig(BATTERY_SBDELAY_NAME, std::bind(&PowerModel::setBatteryScreenBlackDelayModel, m_powerModel, std::placeholders::_1));
+    readConfig(LINEPOWER_LOCKDELAY_NAME, std::bind(&PowerModel::setLinePowerLockDelayModel, m_powerModel, std::placeholders::_1));
+    readConfig(LINEPOWER_SLEEPDELAY_NAME, std::bind(&PowerModel::setLinePowerSleepDelayModel, m_powerModel, std::placeholders::_1));
+    readConfig(LINEPOWER_SBDELAY_NAME, std::bind(&PowerModel::setLinePowerScreenBlackDelayModel, m_powerModel, std::placeholders::_1));
+
+    readConfig(SHOW_HIBERNATE_NAME, std::bind(&PowerModel::setHibernate, m_powerModel, std::placeholders::_1));
+    readConfig(SHOW_SHUTDOWN_NAME, std::bind(&PowerModel::setShutdown, m_powerModel, std::placeholders::_1));
+    readConfig(SHOW_SUSPEND_NAME, std::bind(&PowerModel::setSuspend, m_powerModel, std::placeholders::_1));
+}
+
+void PowerWorker::readConfig(const QString &key, std::function<void(const QVariantList &value)> callback)
+{
+    auto configList = m_cfgPower->value(key).toStringList();
+    bool isLegal = true;
+    if (configList.size() != 6) {
+        isLegal = false;
+    } else {
+        QRegularExpression re("^[1-9][0-9]*[mh]");
+        for (const QString &v : configList) {
+            if (!re.match(v).hasMatch()) {
+                isLegal = false;
+                break;
+            }
+        }
+    }
+    if (!isLegal) {
+        m_cfgPower->reset(key);
+        qWarning() << "config is illegal, reset config";
+    }
+    auto configDataMap = converToDataMap(configList);
+    callback(configDataMap);
+}
+
+void PowerWorker::readConfig(const QString &key, std::function<void(const bool value)> callback)
+{
+    bool enable = m_cfgPower->value(key, true).toBool();
+    callback(enable);
 }
 
 void PowerWorker::deactive()
@@ -153,51 +282,51 @@ void PowerWorker::setSleepOnLidOnPowerClosed(const bool sleep)
 
 void PowerWorker::setSleepDelayOnPower(const int delay)
 {
-    qDebug() << "m_powerDBusProxy->setLinePowerSleepDelay: " << converToDelayDBus(delay);
-    m_powerDBusProxy->setLinePowerSleepDelay(converToDelayDBus(delay));
+    qDebug() << "m_powerDBusProxy->setLinePowerSleepDelay: " << delay;
+    m_powerDBusProxy->setLinePowerSleepDelay(delay);
 }
 
 void PowerWorker::setSleepDelayOnBattery(const int delay)
 {
-    qDebug() << "m_powerDBusProxy->setBatterySleepDelay: " << converToDelayDBus(delay);
-    m_powerDBusProxy->setBatterySleepDelay(converToDelayDBus(delay));
+    qDebug() << "m_powerDBusProxy->setBatterySleepDelay: " << delay;
+    m_powerDBusProxy->setBatterySleepDelay(delay);
 }
 
 void PowerWorker::setScreenBlackDelayOnPower(const int delay)
 {
-    qDebug() << "m_powerDBusProxy->setLinePowerScreenBlackDelay: " << converToDelayDBus(delay);
-    m_powerDBusProxy->setLinePowerScreenBlackDelay(converToDelayDBus(delay));
+    qDebug() << "m_powerDBusProxy->setLinePowerScreenBlackDelay: " << delay;
+    m_powerDBusProxy->setLinePowerScreenBlackDelay(delay);
 }
 
 void PowerWorker::setScreenBlackDelayOnBattery(const int delay)
 {
-    qDebug() << "m_powerDBusProxy->setBatteryScreenBlackDelay: " << converToDelayDBus(delay);
-    m_powerDBusProxy->setBatteryScreenBlackDelay(converToDelayDBus(delay));
+    qDebug() << "m_powerDBusProxy->setBatteryScreenBlackDelay: " << delay;
+    m_powerDBusProxy->setBatteryScreenBlackDelay(delay);
 }
 
 void PowerWorker::setSleepDelayToModelOnPower(const int delay)
 {
-    m_powerModel->setSleepDelayOnPower(converToDelayModel(delay));
+    m_powerModel->setSleepDelayOnPower(delay);
 }
 
 void PowerWorker::setScreenBlackDelayToModelOnPower(const int delay)
 {
-    m_powerModel->setScreenBlackDelayOnPower(converToDelayModel(delay));
+    m_powerModel->setScreenBlackDelayOnPower(delay);
 }
 
 void PowerWorker::setSleepDelayToModelOnBattery(const int delay)
 {
-    m_powerModel->setSleepDelayOnBattery(converToDelayModel(delay));
+    m_powerModel->setSleepDelayOnBattery(delay);
 }
 
 void PowerWorker::setResponseBatteryLockScreenDelay(const int delay)
 {
-    m_powerModel->setBatteryLockScreenDelay(converToDelayModel(delay));
+    m_powerModel->setBatteryLockScreenDelay(delay);
 }
 
 void PowerWorker::setResponsePowerLockScreenDelay(const int delay)
 {
-    m_powerModel->setPowerLockScreenDelay(converToDelayModel(delay));
+    m_powerModel->setPowerLockScreenDelay(delay);
 }
 
 void PowerWorker::setHighPerformanceSupported(bool state)
@@ -287,19 +416,19 @@ bool PowerWorker::getCurCanHibernate()
 
 void PowerWorker::setScreenBlackDelayToModelOnBattery(const int delay)
 {
-    m_powerModel->setScreenBlackDelayOnBattery(converToDelayModel(delay));
+    m_powerModel->setScreenBlackDelayOnBattery(delay);
 }
 
 void PowerWorker::setLockScreenDelayOnBattery(const int delay)
 {
-    qDebug() << "m_powerDBusProxy->setBatteryLockDelay: " << converToDelayDBus(delay);
-    m_powerDBusProxy->setBatteryLockDelay(converToDelayDBus(delay));
+    qDebug() << "m_powerDBusProxy->setBatteryLockDelay: " << delay;
+    m_powerDBusProxy->setBatteryLockDelay(delay);
 }
 
 void PowerWorker::setLockScreenDelayOnPower(const int delay)
 {
-    qDebug() << "m_powerDBusProxy->setLinePowerLockDelay: " << converToDelayDBus(delay);
-    m_powerDBusProxy->setLinePowerLockDelay(converToDelayDBus(delay));
+    qDebug() << "m_powerDBusProxy->setLinePowerLockDelay: " << delay;
+    m_powerDBusProxy->setLinePowerLockDelay(delay);
 }
 
 void PowerWorker::setEnablePowerSave(const bool isEnable)
@@ -307,55 +436,14 @@ void PowerWorker::setEnablePowerSave(const bool isEnable)
     m_powerDBusProxy->setPowerSavingModeEnabled(isEnable);
 }
 
-double PowerWorker::getBatteryCapacity()
-{
-    return m_powerDBusProxy->batteryCapacity();
-}
-
 int PowerWorker::getMaxBacklightBrightness()
 {
     return m_powerDBusProxy->maxBacklightBrightness();
 }
 
-int PowerWorker::converToDelayModel(int value)
+void PowerWorker::setShowBatteryTimeToFull(bool value)
 {
-    if (value == 0) {
-        return 7;
-    }
-
-    if (value <= 60) {
-        return 1;
-    } else if (value <= 300) {
-        return 2;
-    } else if (value <= 600) {
-        return 3;
-    } else if (value <= 900) {
-        return 4;
-    } else if (value <= 1800) {
-        return 5;
-    } else {
-        return 6;
-    }
-}
-
-int PowerWorker::converToDelayDBus(int value)
-{
-    switch (value) {
-    case 1:
-        return 60;
-    case 2:
-        return 300;
-    case 3:
-        return 600;
-    case 4:
-        return 900;
-    case 5:
-        return 1800;
-    case 6:
-        return 3600;
-    case 7:
-        return 0;
-    default:
-        return 900;
+    if (m_cfgDock) {
+        m_cfgDock->setValue("showTimeToFull", value);
     }
 }
